/*++
  The material contained herein is not a license, either        
  expressly or impliedly, to any intellectual property owned    
  or controlled by any of the authors or developers of this     
  material or to any contribution thereto. The material         
  contained herein is provided on an "AS IS" basis and, to the  
  maximum extent permitted by applicable law, this information  
  is provided AS IS AND WITH ALL FAULTS, and the authors and    
  developers of this material hereby disclaim all other         
  warranties and conditions, either express, implied or         
  statutory, including, but not limited to, any (if any)        
  implied warranties, duties or conditions of merchantability,  
  of fitness for a particular purpose, of accuracy or           
  completeness of responses, of results, of workmanlike         
  effort, of lack of viruses and of lack of negligence, all     
  with regard to this material and any contribution thereto.    
  Designers must not rely on the absence or characteristics of  
  any features or instructions marked "reserved" or             
  "undefined." The Unified EFI Forum, Inc. reserves any         
  features or instructions so marked for future definition and  
  shall have no responsibility whatsoever for conflicts or      
  incompatibilities arising from future changes to them. ALSO,  
  THERE IS NO WARRANTY OR CONDITION OF TITLE, QUIET ENJOYMENT,  
  QUIET POSSESSION, CORRESPONDENCE TO DESCRIPTION OR            
  NON-INFRINGEMENT WITH REGARD TO THE TEST SUITE AND ANY        
  CONTRIBUTION THERETO.                                         
                                                                
  IN NO EVENT WILL ANY AUTHOR OR DEVELOPER OF THIS MATERIAL OR  
  ANY CONTRIBUTION THERETO BE LIABLE TO ANY OTHER PARTY FOR     
  THE COST OF PROCURING SUBSTITUTE GOODS OR SERVICES, LOST      
  PROFITS, LOSS OF USE, LOSS OF DATA, OR ANY INCIDENTAL,        
  CONSEQUENTIAL, DIRECT, INDIRECT, OR SPECIAL DAMAGES WHETHER   
  UNDER CONTRACT, TORT, WARRANTY, OR OTHERWISE, ARISING IN ANY  
  WAY OUT OF THIS OR ANY OTHER AGREEMENT RELATING TO THIS       
  DOCUMENT, WHETHER OR NOT SUCH PARTY HAD ADVANCE NOTICE OF     
  THE POSSIBILITY OF SUCH DAMAGES.                              
                                                                
  Copyright 2006, 2007, 2008, 2009, 2010 Unified EFI, Inc. All  
  Rights Reserved, subject to all existing rights in all        
  matters included within this Test Suite, to which United      
  EFI, Inc. makes no claim of right.                            
                                                                
  Copyright (c) 2010, Phoenix Corporation. All rights reserved.   
   
--*/
/*++

Module Name:

  hashes.h

Abstract:
  
  This driver supports platform security service

--*/

#ifndef _UEFI_HASHES_H_
#define _UEFI_HASHES_H_


#define Hex2Ascii(_x)  ( ( ((_x)&0x0f)<10) ? ('0'+((_x)&0x0f)) : ('a'-10+((_x)&0x0f)) )
#define HEX2ASCII(_x)  ( ( ((_x)&0x0f)<10) ? ('0'+((_x)&0x0f)) : ('A'-10+((_x)&0x0f)) )

#define ASCII2HEX(_x)   ( ((_x)<='9') ? ((_x)-'0') : (((_x)<='F') ? ((_x)-'A') : ((_x)-'a')) )
#define Ascii2Hex ASCII2HEX

#define IN 
#define OUT 
#define INOUT 

#ifndef _EFI_TYPES_H_
#define EFIAPI 
typedef struct EFI_GUID {
  UINT32  Data1;
  UINT16  Data2;
  UINT16  Data3;
  UINT8   Data4[8];
} EFI_GUID;
#endif

#define EFI_HASH_ALGORITHM_SHA1_GUID 	{0x2ae9d80f, 0x3fb2, 0x4095, {0xb7,0xb1,0xe9,0x31,0x57,0xb9,0x46,0xb6}}
#define EFI_HASH_ALGORITHM_SHA224_GUID 	{0x8df01a06, 0x9bd5, 0x4bf7, {0xb0,0x21,0xdb,0x4f,0xd9,0xcc,0xf4,0x5b}}
#define EFI_HASH_ALGORITHM_SHA256_GUID 	{0x51aa59de, 0xfdf2, 0x4ea3, {0xbc,0x63,0x87,0x5f,0xb7,0x84,0x2e,0xe9}}
#define EFI_HASH_ALGORITHM_SHA384_GUID 	{0xefa96432, 0xde33, 0x4dd2, {0xae,0xe6,0x32,0x8c,0x33,0xdf,0x77,0x7a}}
#define EFI_HASH_ALGORITHM_SHA512_GUID 	{0xcaa4381e, 0x750c, 0x4770, {0xb8,0x70,0x7a,0x23,0xb4,0xe4,0x21,0x30}}
#define EFI_HASH_ALGORITHM_MD5_GUID 	{0x0af7c79c, 0x65b5, 0x4319, {0xb0,0xae,0x44,0xec,0x48,0x4e,0x4a,0xd7}}

#define HASHSIZE_MD5 	16
#define HASHSIZE_SHA1 	20
#define HASHSIZE_SHA224 28
#define HASHSIZE_SHA256 32
#define HASHSIZE_SHA384 48
#define HASHSIZE_SHA512 64
#define HASHSIZE_MAXIUM HASHSIZE_SHA512

#define GUID_STRING_SIZE 16

typedef UINT8 EFI_MD5_HASH[HASHSIZE_MD5];
typedef UINT8 EFI_SHA1_HASH[HASHSIZE_SHA1];
typedef UINT8 EFI_SHA224_HASH[HASHSIZE_SHA224];
typedef UINT8 EFI_SHA256_HASH[HASHSIZE_SHA256];
typedef UINT8 EFI_SHA384_HASH[HASHSIZE_SHA384];
typedef UINT8 EFI_SHA512_HASH[HASHSIZE_SHA512];


typedef union _EFI_HASH_OUTPUT {
	EFI_MD5_HASH *Md5Hash;
	EFI_SHA1_HASH *Sha1Hash;
	EFI_SHA224_HASH *Sha224Hash;
	EFI_SHA256_HASH *Sha256Hash;
	EFI_SHA384_HASH *Sha384Hash;
	EFI_SHA512_HASH *Sha512Hash;
} EFI_HASH_OUTPUT;

// {C5184932-DBA5-46DB-A5BA-CC0BDA9C1435}
#define  EFI_HASH_PROTOCOL_GUID \
	{0xc5184932, 0xdba5, 0x46db, 0xa5,0xba,0xcc,0x0b,0xda,0x9c,0x14,0x35};	

EFI_FORWARD_DECLARATION (EFI_HASH_PROTOCOL);

typedef EFI_STATUS EFI_HASH_GET_HASH_SIZE(  IN  EFI_HASH_PROTOCOL 	*this,
									IN  EFI_GUID 		*HashAlgorithm,	
									OUT UINTN 		*HashSize );

typedef EFI_STATUS EFI_HASH_HASH( IN  EFI_HASH_PROTOCOL 	*this,
							IN EFI_GUID			*HashAlgorithm,
							IN BOOLEAN Extend,
							IN UINT8 *Message,
							IN UINT64 MessageSize,
							IN OUT UINT8/*EFI_HASH_OUTPUT*/ *Hash );	
	
typedef struct _EFI_HASH_PROTOCOL {
	EFI_HASH_GET_HASH_SIZE *GetHashSize;
	EFI_HASH_HASH *Hash;	
} EFI_HASH_PROTOCOL;

/* MD5 context. */
typedef struct {
  UINT32 state[4];        /* state (ABCD) */
  UINT32 count[2];        /* number of bits, modulo 2^64 (lsb first) */
  UINT8  buffer[64];      /* input buffer */
} MD5_CTX;

typedef struct {
  char *hash;
  UINT32 D[5], M[16], X[5], mlen[2];
  UINT16 state;
} sha1_t;

typedef struct {
	UINT32 state[8], length, curlen;
	unsigned char buf[64];
} sha256_state;
#define sha224_state sha256_state

typedef struct {
   UINT64 state [8], length, curlen;
   unsigned char buf [128];
   } sha384_state;

#define sha512_state sha384_state


typedef union _EFI_HASH_STATE {
	MD5_CTX       md5;
	sha1_t		sha1;
	sha256_state sha256;
	sha512_state sha512;
} EFI_HASH_STATE;

#define EFI_HASH_SERVICE_BINDING_PROTOCOL_GUID {0x42881c98, 0xa4f3, 0x44b0, 0xa3,0x9d,0xdf,0xa1,0x86,0x67,0xd8,0xcd}	


int getHashResultSize(EFI_GUID *hashGuid);
int getHashResult(EFI_HASH_STATE *md, EFI_GUID *hashGuid, int hashExtend, unsigned char *message, int messageSize, unsigned char *hashOutput);								
#endif
